var http = require('http');
var url=require('url');
var fs = require('fs');
var sys= require('sys');

process.env['NODE_TLS_REJECT_UNAUTHORIZED'] = '0';

function s4() {
  return Math.floor((1 + Math.random()) * 0x10000)
             .toString(16)
             .substring(1);
};

function guid() {
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
         s4() + '-' + s4() + s4() + s4();
}

function loadData(){
	var json=fs.readFileSync("etc/config.txt", "utf-8");
	return JSON.parse(json);
}

function getServerData(){
	var content=fs.readFileSync("/usr/etc/lighttpd/setting.conf", "utf-8");
	var lines=content.split(/\n/);
	var result={}
	lines.forEach(function(line){
		var res=line.match(/.*var\.http_port.*=[ ]*([0-9]+)/);
		if (res) result.http=res[1];
		var res=line.match(/.*var\.https_port.*=[ ]*([0-9]+)/);
		if (res) result.https=res[1];
	});
	return(result);
}

function saveData(data){
	data.sort(function(a,b){a.pc < b.pc});
	fs.writeFileSync("etc/config.txt", JSON.stringify(data));
}

function standardView(){

	var html=fs.readFileSync("webman/template.html", "utf-8");
	
	var data=loadData();
	
	var output="";
	
	if (data.length>0){
		var json=JSON.stringify(data);
		output=html.replace(/###DATA###/,"var dataArr="+json+";");
	}
	else output=html.replace(/###DATA###/,"var dataArr=[];");
	return output;
}

function headerView(){
	var html=fs.readFileSync("webman/header.html", "utf-8");
	return html;
}

function errorView(message){
	
	var output="There was an unspecified error.";
	if (message) output=message;
	return '<div id="error"><fieldset><legend>Error</legend>'+output+'</fieldset></div></body></html>';		
}


function wake(queryData){
	
	var uid=queryData.uid;
	var data=loadData();
	var mac=null;
	for (var i=0; i<data.length; i++){
		if (data[i].uid==uid) mac=data[i].mac;
	}
	
	if (mac){
		var exec = require('child_process').exec;
		var child;
		child=exec('ether-wake -i eth0 '+mac,function(error,stdout,stderr){		
		});
		child=exec('ether-wake -i eth1 '+mac,function(error,stdout,stderr){		
		});
	}
}

function del(queryData)
{
	//var pc=queryData.pc;
	//var mac=queryData.mac;
	//var ip=queryData.ip;
	var uid=queryData.uid;
	
	var data=loadData();
	var key=queryData.key;
	
	var newData=[];
	
	for (var i=0; i<data.length; i++){
		//if (!data[i].ip) data[i].ip="";
		if (data[i].uid!=uid) newData.push(data[i]);
	}

	saveData(newData);
}

function add(queryData){

	var mac=queryData.mac;
	var pc=queryData.pc;
	var ip=queryData.ip;
	
	var data=loadData();
	data.push({pc:pc,mac:mac,ip:ip,uid:guid()});
	
	data.sort(function(a,b){return a.pc>b.pc});
	
	saveData(data);

}

function checkstate(queryData,res){
	var ip=null;
	var uid=queryData.uid;
	
	var data=loadData();
	for (var i=0; i<data.length; i++){
		//if (!data[i].ip) data[i].ip="";
		if (data[i].uid==uid) ip=data[i].ip;
	}
	
	var exec = require('child_process').exec;
	var child;
	child=exec('ping -c 1 -w 1 '+ip+' | grep time',function(error,stdout,stderr){					
			if (stdout) res.write("on");
			else res.write("off");
			res.end();
	});
	
}


function outputError(res,message){
	
	var content=headerView();
	content+=errorView(message);
		
	res.writeHead(200, {'Content-Type': 'text/html'});
	res.write(content);
	res.end();
}

function output(res){
	
	var content=headerView();
	content+=standardView();
	
	res.writeHead(200, {'Content-Type': 'text/html'});
	res.write(content);
	res.end();
}

var serverData=getServerData();

http.createServer(function (req, res) {
	
	var uri = url.parse(req.url).pathname;
		
	if (uri.match(/^\/images/)){
		res.writeHead(200, {'Content-Type':"image/png"});

		var filename = process.cwd()+"/webman"+uri.replace('/\.\.\/g','');
		
		if (fs.existsSync(filename)) {
			var fileStream = fs.createReadStream(filename);
			fileStream.pipe(res);
			return;
		}		
        
	}
	
	
	if (!req.headers.cookie || req.headers.cookie.indexOf("as_sid") <0) {
		outputError(res);
		return;
	}
	var result=req.headers.cookie.match(/.*as_sid=(.*?)[;^]/);
	if (!result) result=req.headers.cookie.match(/.*as_sid=(.*)/);

	var sid=result[1];
	
	var auth_url='';
	var client;
	if (serverData.http) {
		auth_url='http://localhost:'+serverData.http;
		client=require('http');
	}
	else if (serverData.https){ 
		auth_url='https://localhost:'+serverData.https;
		client=require('https');
	}
	else {
		outputError(res,"Can't detect the port the NAS is listening to.");
		return;
	}
	
	
	var queryData = url.parse(req.url, true).query;
	
	if (Object.keys(queryData).indexOf("checkstate")>=0) {
		checkstate(queryData,res);
		return;
	}
	
	var auth_url=auth_url+'/portal/apis/information/onlineuser.cgi?sid='+sid+'&act=list&sort=0&ascend=0&start=0&limit=-1';

	
	client.get(auth_url, function(response) {
		
			var answer=''
	
			response.on("data", function(chunk) {
				answer+=chunk;
			});
			
			response.on('end', function(){

				var answerObj=JSON.parse(answer);
								
				if (answerObj.success && answerObj.total && answerObj.total>0){
			
					if (Object.keys(queryData).indexOf("wake")>=0) {
						wake(queryData); 
						output(res);
					}	
					else if (Object.keys(queryData).indexOf("del")>=0) {
						del(queryData); 
						output(res);
					}
					else if (Object.keys(queryData).indexOf("add")>=0) {
						add(queryData); 
						output(res);
					}
					else output(res);			
				}
				else outputError(res);
			
			});
			
	
	}).on('error', function(e) {
		outputError(res);
	});

	
}).listen(9876, '0.0.0.0');
//console.log('Server running at http://0.0.0.0:9876/');
